#define INCL_WIN
#define INCL_ERRORS
#define INCL_BASE
#include <os2.h>

#include <stdio.h>
#include <string.h>
#include <math.h>

/****************************************************************************
   This header file contains all pragmas, function prototypes, defines, and
   structure definitions.
****************************************************************************/
#include "plugin.h"

/****************************************************************************
   This is the plugins initilization function.  It is called once, when
   ColorWorks is first started.  This function must appear in your plugin.
   The "EffectEntryPoint", "EffectName", and the "UndoName" members of the
   "PluginInitilization" structure MUST! be filled in by this function.
   Allocating a static data area is optional.   A return of anything but
   "NO_ERROR" from this function indicates an error occured initilizing the
   plugin.  If an error is returned from this function, the plugin will not be
   made available to the user.  It is the responsibility of the plugin to
   inform the user that an error occured initilizing the plugin.
   Take special note of the second parameter being passed to the "WinLoadString"
   function.  This is the handle to the DLL being initilized.  When
   loading resources, such as a string, make sure you load the resource
   from the DLL using the DLL's handle. If you dont, and you use NULLHANDLE
   instead, the resource will attempt to be loaded from the .EXE instead of
   from your DLL. An ERROR for sure!  This function must be compiled using
   the _System linkage, see top of file "plugin.h" for more information.
****************************************************************************/
INT InitilizePlugin(struct _PluginInitilization *PluginInitilization)
{
   struct _MotionBlurDialogSettings *MotionBlurDialogSettings;
   CHAR   ErrorString[CCHMAXPATH];

 /* This is an effect plugin and belongs in the "Plugin Efects" submenu off
    the "Edit" menu.  ColorWorks needs to be informed of the catagory of
    a particular plugin DLL in order to place it in the correct menu and
    properly process the plugin when selected by the user. */
   PluginInitilization->PluginCatagory = PLUGINEFFECT;
 /* This is the name of the plugins entry point function, this is the function
    ColorWorks will call when the user selects this plugin from the menu. */
   strcpy(PluginInitilization->EffectEntryPoint, "PluginEntryPoint");
 /* This is the name of the plugin as it appears in the ColorWorks menu. */
   strcpy(PluginInitilization->EffectName, "Motion Blur...");
 /* This is the text which will appear in the "Undo/Redo Float" menu item in
    the "Edit" menu. */
   strcpy(PluginInitilization->UndoName, "Motion Blur");
 /* Each time a user enters the motion blur dialog the settings of the dialogs
    controls should reflect the state of the controls the last time the user
    used the motion blur function.  This is simply good user interface design.
    To facilitate this the plugin interface allows the plugin designer to
    allocate an area of memory during initilization of the plugin.  This area
    of memory will be maintained by ColorWorks and will be made available to the
    plugin each time it is called.  The "StaticData" member of the
    "RuntimeStructure" structure is where the memory pointer will be placed for
    the plugins use. This memory can be used for any purpose the plugin designer
    sees fit. The next five lines of code allocate this static data area,
    loads its members with default values, and then sets the "StaticData"
    member of the "PluginInitilization" structure to point to the memory. */
   MotionBlurDialogSettings = (struct _MotionBlurDialogSettings *)malloc(sizeof(struct _MotionBlurDialogSettings));
   if(MotionBlurDialogSettings == (struct _MotionBlurDialogSettings *)NULL)
   {
      WinLoadString(PluginInitilization->HwndAB, PluginInitilization->ModuleHandle, STRING1, sizeof(ErrorString), ErrorString);
      WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PSZ)ErrorString, (PSZ)NULL,
                    (USHORT)NULL, MB_ERROR | MB_OK | MB_MOVEABLE | MB_APPLMODAL);
      return(ERROR);
   }
   MotionBlurDialogSettings->HorizontalDirection = MOVE_RIGHT;
   MotionBlurDialogSettings->VerticalDirection   = NO_MOVEMENT;
   MotionBlurDialogSettings->BlurVelocity        = 10;
   PluginInitilization->StaticData = (void *)MotionBlurDialogSettings;

   return(NO_ERROR);
}

/****************************************************************************
   This is the function ColorWorks calls when the user selects the plugin
   effect "Motion Blur..." from the ColorWorks menu.  The one parameter is a
   pointer to the "_PluginRuntime" structure.  This structure contains the
   information necessary to perform the effect.  This function must be
   compiled using the _System linkage, as opposed to VisualAge's default
   _Optlink.  See top of file "plugin.h" for more information.
****************************************************************************/
VOID PluginEntryPoint(struct _PluginRuntime *PluginRuntime)
{
   LoadMotionBlurDialog(PluginRuntime);
}

/**************************************************************************
   This function loads the motion blur dialog.  Take special note of the
   fourth parameter being passed to the "WinDlgBox" function.  This is
   the handle to the DLL being used to execute the effect.  When loading
   resources, such as a dialog, make sure you load the resource from the DLL
   using the DLL's handle. If you dont, and you use NULLHANDLE instead, the
   resource will attempt to be loaded from the .EXE instead of from your DLL.
   An ERROR for sure!
**************************************************************************/
VOID LoadMotionBlurDialog(struct _PluginRuntime *RuntimeStructure)
{
   HWND    HwndCWClientWindow;
   HMODULE DLLModuleHandle;
   ULONG   rc;

   HwndCWClientWindow = RuntimeStructure->HwndCWClientWindow;
   DLLModuleHandle    = RuntimeStructure->ModuleHandle;
   rc = WinDlgBox(HWND_DESKTOP,    HwndCWClientWindow, (PFNWP)MotionBlurDialogProcedure,
                  DLLModuleHandle, MOTIONBLUR_DIALOG,  (PVOID)RuntimeStructure);
}

/**************************************************************************
   This is the dialog procedure for the motion blur dialog.  This
   function is mostly a dispatch routine.  It does perform a small amount
   of processing for some messages, but nothing big.
**************************************************************************/
MRESULT EXPENTRY MotionBlurDialogProcedure(HWND   HwndDialog, ULONG  message,
                                           MPARAM param1,     MPARAM param2)
{
   struct _MotionBlurDialogSettings *MotionBlurDialogSettings;
   struct _PluginRuntime *PluginRuntime;

   PluginRuntime = (struct _PluginRuntime *)WinQueryWindowPtr(HwndDialog, QWL_USER);
   switch( message )
   {
      case WM_INITDLG :
         InitilizeMotionBlurDialog(HwndDialog, message, param1, param2);
         return( (MRESULT)TRUE );

      case WM_CLOSE :
         message = WM_COMMAND;
         param1  = MPFROM2SHORT(DID_CANCEL, (SHORT2FROMMP(param1)));
         WinPostMsg(HwndDialog, message, param1, (MPARAM)NULL);
         return( (MRESULT)TRUE );

      case WM_COMMAND :
         switch(SHORT1FROMMP(param1))
         {
/* The user clicked the "OK" pushbutton. */
            case DID_OK :
               WinDismissDlg(HwndDialog, (ULONG)NULL);
               FreeMotionBlurDialogResources(PluginRuntime);
               LoadSystemPointer(SPTR_WAIT);
               MotionBlurDialogSettings = (struct _MotionBlurDialogSettings *)PluginRuntime->StaticData;
               MotionBlurDialogSettings->BlurVelocity = (char)QueryNumericSpinButton(HwndDialog, VELOCITY_SPINNER);
               GetMotionBlurDirection(HwndDialog, &(MotionBlurDialogSettings->HorizontalDirection), &(MotionBlurDialogSettings->VerticalDirection));
               ApplyMotionBlur(PluginRuntime, MotionBlurDialogSettings->BlurVelocity, MotionBlurDialogSettings->HorizontalDirection, MotionBlurDialogSettings->VerticalDirection);
               LoadSystemPointer(SPTR_ARROW);
               return( (MRESULT)DID_OK );

/* The user clicked the "Cancel" pushbutton. */
            case DID_CANCEL :
               FreeMotionBlurDialogResources(PluginRuntime);
               WinDismissDlg(HwndDialog, (ULONG)NULL);
               return( (MRESULT)DID_CANCEL );

/* The user clicked the "Preview" pushbutton. */
            case DID_PREVIEW :
               MotionBlurDialogSettings = (struct _MotionBlurDialogSettings *)PluginRuntime->StaticData;
               WinPostMsg(MotionBlurDialogSettings->HwndPreviewWindowClient,  WM_PAINT, (MPARAM)NULL, (MPARAM)NULL);
               return((MRESULT)TRUE);
         }
         break;

      default:  return( WinDefDlgProc(HwndDialog, message, param1, param2) );
   }
}

/**************************************************************************
   This function initilizes the motion blur dialog, it is called when the
   motion blur dialog procedure recieves the "WM_INITDLG" message.  This
   function sets the dialogs window pointer to point to the "PluginRuntime"
   structure, sets the dialogs controls to their default values, and positions
   and activates the dialog.  Also, The velocity spinner control is set
   to have the keyboard focus.
**************************************************************************/
VOID InitilizeMotionBlurDialog(HWND   HwndDialog, ULONG  message,
                               MPARAM param1,     MPARAM param2)
{
   struct _MotionBlurDialogSettings *MotionBlurDialogSettings;
   struct _PluginRuntime *PluginRuntime;
   HWND   Hwnd;
   INT    i;

   PluginRuntime = (struct _PluginRuntime *)param2;
 /* Get the pointer to the "MotionBlurDialogSettings" structure. */
   MotionBlurDialogSettings = (struct _MotionBlurDialogSettings *)PluginRuntime->StaticData;
 /* Set the user window pointer of the dialog to point to the "PluginRuntime" structure. */
   WinSetWindowPtr(HwndDialog, QWL_USER, (PVOID)PluginRuntime);

 /* Initilize the velocity spin button. */
   WinSendDlgItemMsg(HwndDialog, VELOCITY_SPINNER, SPBM_SETLIMITS,       MPFROMLONG(255L), MPFROMLONG(1L));
   WinSendDlgItemMsg(HwndDialog, VELOCITY_SPINNER, SPBM_SETCURRENTVALUE, (MPARAM)MotionBlurDialogSettings->BlurVelocity,       (MPARAM)NULL);
   WinSendDlgItemMsg(HwndDialog, VELOCITY_SPINNER, SPBM_SETTEXTLIMIT,    (MPARAM)3,        (MPARAM)NULL);

 /* Initilize the radiobuttons in the dialog. */
   if(MotionBlurDialogSettings->HorizontalDirection == NO_MOVEMENT)
      WinSendDlgItemMsg(HwndDialog, HORIZ_NONE_RADIO, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(MotionBlurDialogSettings->HorizontalDirection == MOVE_RIGHT)
      WinSendDlgItemMsg(HwndDialog, HORIZ_RIGHT_RADIO, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(MotionBlurDialogSettings->HorizontalDirection == MOVE_LEFT)
      WinSendDlgItemMsg(HwndDialog, HORIZ_LEFT_RADIO, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);

   if(MotionBlurDialogSettings->VerticalDirection == NO_MOVEMENT)
      WinSendDlgItemMsg(HwndDialog, VERT_NONE_RADIO,   BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(MotionBlurDialogSettings->VerticalDirection == MOVE_DOWN)
      WinSendDlgItemMsg(HwndDialog, VERT_DOWN_RADIO,   BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(MotionBlurDialogSettings->VerticalDirection == MOVE_UP)
      WinSendDlgItemMsg(HwndDialog, VERT_UP_RADIO,   BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);

 /* Center the motion blur dialog on screen. */
   (*(PFN)PluginRuntime->CallbackFunctions.CenterDialogBox)(HwndDialog);
 /* Set the motion blur dialog as the active window. */
   WinSetActiveWindow(HWND_DESKTOP, HwndDialog);
 /* Give the velocity spiner the keyboard focus. */
   WinSetFocus(HWND_DESKTOP, WinWindowFromID(HwndDialog, VELOCITY_SPINNER));

 /* Create a preview window. */
   Hwnd = (*(PFN)PluginRuntime->CallbackFunctions.CreatePreviewWindow)(HwndDialog, PREVIEW_FRAME,
                                                                (PFNWP)PreviewWindowProcedure, PluginRuntime);
 /* Set the frame and client window handle variables of the "MotionBlurDialogSettings"
    structure. */
   MotionBlurDialogSettings->HwndPreviewWindowFrame  = Hwnd;
   MotionBlurDialogSettings->HwndPreviewWindowClient = WinWindowFromID(Hwnd, FID_CLIENT);
 /* Set the user window pointer of the preview window to point to the "PluginRuntime" structure. */
   WinSetWindowPtr(MotionBlurDialogSettings->HwndPreviewWindowClient, QWL_USER, PVOIDFROMMP(PluginRuntime));
 /* Make the preview window visible. */
   WinShowWindow(MotionBlurDialogSettings->HwndPreviewWindowFrame, TRUE);
}

/**************************************************************************
    This function releases any resources required by the motion blur dialog.
**************************************************************************/
VOID FreeMotionBlurDialogResources(struct _PluginRuntime *PluginRuntime)
{
   struct _MotionBlurDialogSettings *MotionBlurDialogSettings;

   MotionBlurDialogSettings = (struct _MotionBlurDialogSettings *)PluginRuntime->StaticData;
   WinDestroyWindow(MotionBlurDialogSettings->HwndPreviewWindowFrame);
}

/*************************************************************************
   This function performs the task of actualy applying the motion blur to
   the float.
*************************************************************************/
VOID ApplyMotionBlur(struct _PluginRuntime *PluginRuntime, CHAR BlurVelocity,
                     INT HorizontalDirection, INT VerticalDirection)
{
   INT   FloatRowsizeInBytes, FloatRowPointer;
   INT   FloatWidth, FloatHeight;
   INT   Red, Green, Blue;
   INT   i, j, jo, n;
   INT   Completion;
   INT   x, y;
   UCHAR *RowPointer, *PixelPointer;
   UCHAR *NewFloatPixelPointer;
   UCHAR *TestTransparencyPixelPointer;
   UCHAR *NewFloatPointer;
   UCHAR *FloatedImage;
   VOID  *HwndCanvas;

/* Set local variables, this is done for readability, shorter variable names. */
   HwndCanvas          = PluginRuntime->HwndCanvas;
   FloatWidth          = PluginRuntime->FloatData.Width;
   FloatHeight         = PluginRuntime->FloatData.Height;
   FloatRowsizeInBytes = PluginRuntime->FloatData.RowsizeInBytes;
   FloatedImage        = PluginRuntime->FloatData.ImageDataPointer;

/* This function returns a pointer to a new float of the same dimentions as
   the original float, the original float is not, and should never, be freed. */
   NewFloatPointer = CreateANewFloat(PluginRuntime, FloatWidth, FloatHeight);
/* If the new float could not be created their is no point is proceeding. */
   if(NewFloatPointer == (UCHAR *)NULL)
      return;
/* Now that we are sure that the application of our effect will be sucessful we
   save an undo of the float. */
   (*(PFN)PluginRuntime->CallbackFunctions.SaveTransformUndo)();

   if(VerticalDirection*HorizontalDirection == MOVEMENT_IN_BOTH_DIRECTIONS)
      BlurVelocity = BlurVelocity * 10 / 14;

/* Process every row of the float. */
   for(i=0; i<FloatHeight; i++)
   {
/* Calculate the precentage of the effect which has completed. */
      Completion = ((float)i / (float)FloatHeight) * 100.0;
/* Set the completion ribbon to reflect the precentage of completion. */
      (*(PFN)PluginRuntime->CallbackFunctions.SetCompletionRibbon)(Completion, HwndCanvas);

/* Process every column of the float. */
      for(j=0; j<FloatWidth; j++)
      {
/* Set variables to their initial values. */
         Red = 0; Green = 0; Blue = 0; n = BlurVelocity;
/* Motion blur the number of pixels as set by "BlurVelocity". */
         for(jo=0; jo<BlurVelocity; jo++)
         {
/* Compute grid location to get next bluring pixel from. */
            x = j + HorizontalDirection * jo;
            y = i + VerticalDirection   * jo;
/* Check if the location of the next bluring pixel is within the bounds of the
   float. */
            if(y >= 0 && y < FloatHeight && x >= 0 && x < FloatWidth)
            {
/* Check to see if the pixel which is about to be added to the blur computation
   is a transparant pixel.  If it is, dont use it.  Transparant pixels should
   be ignored and should not be used in any computations. */
               RowPointer = FloatedImage + (y * FloatRowsizeInBytes);
               TestTransparencyPixelPointer = RowPointer + (x * BYTES_PER_PIXEL);
               if(IsPixelTransparant(TestTransparencyPixelPointer, PluginRuntime) == NO)
               {
/* Set rowpointer to point to the row of the next bluring pixel. */
                  RowPointer = FloatedImage + (y * FloatRowsizeInBytes);
/* Index into the row to point to the next bluring pixel. */
                  PixelPointer = RowPointer + (x * BYTES_PER_PIXEL);
               }
            }
/* Add the bluring pixel to the current Red, Green, Blue component values,
   multiplied by current bluring location. */
            Blue  += n * *PixelPointer;
            Green += n * *(PixelPointer + 1);
            Red   += n * *(PixelPointer + 2);
            n--;
         }
/* Blur the pixel. */
         Blue  = Blue  / ((BlurVelocity+1) * BlurVelocity / 2);
         Green = Green / ((BlurVelocity+1) * BlurVelocity / 2);
         Red   = Red   / ((BlurVelocity+1) * BlurVelocity / 2);

/* Make sure that the color components of the blured pixel are within the
   allowed bounds, 0 to 255. If they are not, snap them in bounds. */
         CHECKPIXELBOUNDS(Blue);
         CHECKPIXELBOUNDS(Green);
         CHECKPIXELBOUNDS(Red);

/* Set the row pointer to point to the row of the pixel to be set in the new
   float. */
         RowPointer = NewFloatPointer + (i * FloatRowsizeInBytes);
/* Index into the row to point to the pixel to be set. */
         NewFloatPixelPointer = RowPointer + (j * BYTES_PER_PIXEL);

/* Check to see if the pixel to be set is transparant.  If it is dont set
   the pixel, leave it as transparant. */
         if(IsPixelTransparant(NewFloatPixelPointer, PluginRuntime) == NO)
         {
/* Make sure the color of the blured pixel is not transparant, and if it
   is, offset it so it will not be. */
            ScrubColor(&Blue, &Green, &Red, PluginRuntime);
/* Set the pixel in the new float to the blur pixel. */
            *NewFloatPixelPointer     = Blue;
            *(NewFloatPixelPointer+1) = Green;
            *(NewFloatPixelPointer+2) = Red;
         }
      }
   }
/* The motion blur is complete, so we reset the completion ribbon. */
   (*(PFN)PluginRuntime->CallbackFunctions.SetCompletionRibbon)(TASK_COMPLETED, HwndCanvas);

/* Inform colorWorks that a new float was created. */
   (*(PFN)PluginRuntime->CallbackFunctions.NewFloatWasCreated)(NewFloatPointer, FloatWidth, FloatHeight);
/* Merge the float with the videobuffer, making the results of the motion blur
   visible to the user. */
   (*(PFN)PluginRuntime->CallbackFunctions.MergeFloatWithVideobuffer)(HwndCanvas);
}

/**************************************************************************
   This function querys and returns the value of a numeric spin button.
**************************************************************************/
LONG QueryNumericSpinButton(HWND HwndDialog, ULONG SpinButtonID)
{
   LONG SpinButtonValue;

   WinSendDlgItemMsg(HwndDialog, SpinButtonID, SPBM_QUERYVALUE, MPFROMP(&SpinButtonValue),
                     MPFROM2SHORT((SHORT)NULL, SPBQ_ALWAYSUPDATE) );
   return(SpinButtonValue);
}

/**************************************************************************
   This function querys and returns the checkstate of a radiobutton.
**************************************************************************/
USHORT IsRadioButtonChecked(HWND HwndDialog, ULONG RadioButtonID)
{
   USHORT CheckState;

   CheckState = (USHORT)WinSendDlgItemMsg(HwndDialog, RadioButtonID, BM_QUERYCHECK,
                                          (MPARAM)NULL, (MPARAM)NULL);
   return(CheckState);
}

/**************************************************************************
   This function querys the horizontal direction radio buttons and the
   vertical direction radio buttons and sets the "HorizontalDirection" and
   "VerticalDirection" variables to the appropriate values.
**************************************************************************/
VOID GetMotionBlurDirection(HWND HwndDialog, INT *HorizontalDirection,
                            INT *VerticalDirection)
{
   USHORT CheckState;

   CheckState = IsRadioButtonChecked(HwndDialog, HORIZ_RIGHT_RADIO);
   if(CheckState == CHECKED)
      *HorizontalDirection = MOVE_RIGHT;
   CheckState = IsRadioButtonChecked(HwndDialog, HORIZ_LEFT_RADIO);
   if(CheckState == CHECKED)
      *HorizontalDirection = MOVE_LEFT;
   CheckState = IsRadioButtonChecked(HwndDialog, HORIZ_NONE_RADIO);
   if(CheckState == CHECKED)
      *HorizontalDirection = NO_MOVEMENT;

   CheckState = IsRadioButtonChecked(HwndDialog, VERT_UP_RADIO);
   if(CheckState == CHECKED)
      *VerticalDirection = MOVE_UP;
   CheckState = IsRadioButtonChecked(HwndDialog, VERT_DOWN_RADIO);
   if(CheckState == CHECKED)
      *VerticalDirection = MOVE_DOWN;
   CheckState = IsRadioButtonChecked(HwndDialog, VERT_NONE_RADIO);
   if(CheckState == CHECKED)
      *VerticalDirection = NO_MOVEMENT;

}

/**************************************************************************
   A floated image can be any shape, and floats which are not rectangular
   have areas which are transparant.  These transparant portions of the float
   have a predefined pixel value that ColorWorks recognizes as transparant,
   and handles as such when manipulating the float.  Typicaly, an effect,
   such as motion blur, needs to know if a pixel is transparant or not.
   Normally the effect is not applied to the transparant pixels.  The job
   of this function is to return to the caller wether or not a pixel is
   transparant.
**************************************************************************/
BOOL IsPixelTransparant(UCHAR *PixelPointer, struct _PluginRuntime *PluginRuntime)
{
/* If the pixel is transparent return "YES" Otherwise return "NO". */
   if(*PixelPointer     == PluginRuntime->TransparentColor.TransparentBlue  &&
      *(PixelPointer+1) == PluginRuntime->TransparentColor.TransparentGreen &&
      *(PixelPointer+2) == PluginRuntime->TransparentColor.TransparentRed)
         return(YES);
      else
         return(NO);
}

/**************************************************************************
   This function loads and sets an OS/2 system pointer.
**************************************************************************/
VOID LoadSystemPointer(LONG PointerID)
{
   HPOINTER HSystemPointer;

   HSystemPointer = WinQuerySysPointer(HWND_DESKTOP, PointerID, FALSE);
   WinSetPointer(HWND_DESKTOP, HSystemPointer);
}

/**************************************************************************
   When manipulating the values of a pixel, such as in computing a motion blur,
   it is possible to convert a pixel to a transparant value.  If this occures,
   the computed pixel will be transparant when the floated image is merged
   with the canvas.  This function rectifies this situation should this occure.
   It does so by modulating the blue component of the pixel by one value,
   if the blue component is equal to the blue transparent value.
**************************************************************************/
VOID ScrubColor(INT *Blue, INT *Green, INT *Red, struct _PluginRuntime *PluginRuntime)
{
/* If the blue component of the pixel is transparent increment the blue
   component by one. */
   if(*Blue == PluginRuntime->TransparentColor.TransparentBlue)
      (*Blue)++;
}


/**************************************************************************
   This function allocates memory to create a new float.  If the allocation
   is sucessfull the contents of the original float is copied, via memcpy,
   to the new float.  A pointer to the new float is returned to the caller.
   If the allocation fails the user is notified through a message box.
**************************************************************************/
UCHAR *CreateANewFloat(struct _PluginRuntime *PluginRuntime,
                       INT FloatWidth, INT FloatHeight)
{
   CHAR  ErrorString[CCHMAXPATH];
   UCHAR *NewFloatPointer;
   INT   FloatSizeInBytes;

/* Compute the floats new size in bytes. */
   FloatSizeInBytes = (FloatWidth * BYTES_PER_PIXEL) * FloatHeight;
/* Allocate a memory block large enough for the new float. */
   NewFloatPointer  = (UCHAR *)malloc(FloatSizeInBytes);
/* Error process if the memory allocation faild. */
   if(NewFloatPointer == (UCHAR *)NULL)
   {
/* Load the error string from the DLL resource. */
      WinLoadString(PluginRuntime->HwndAB, PluginRuntime->ModuleHandle, STRING2, sizeof(ErrorString), ErrorString);
/* Display an error message box with the previously loaded message. */
      WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PSZ)ErrorString, (PSZ)NULL,
                    (USHORT)NULL, MB_ERROR | MB_OK | MB_MOVEABLE | MB_APPLMODAL);
   }
   else
/* Copy the floated image to the new memory block. */
      memcpy(NewFloatPointer, PluginRuntime->FloatData.ImageDataPointer, FloatSizeInBytes);

   return(NewFloatPointer);
}


/**************************************************************************
   This is the window procedure for the preview window in the dialog.
   When the user selects the "Preview" pushbutton a WM_PAINT message is
   posted to this window procedure.
**************************************************************************/
MRESULT EXPENTRY PreviewWindowProcedure(HWND HwndClient, ULONG message, MPARAM param1, MPARAM param2)
{
   struct _PluginRuntime *PluginRuntime;
   VOID   *HwndCanvas;

/* Get the pointer to the PluginRuntime structure. */
   PluginRuntime = (struct _PluginRuntime *)WinQueryWindowPtr(HwndClient, QWL_USER);

   switch(message)
   {
      case WM_PAINT :
/* display the hour wait pointer. */
         LoadSystemPointer(SPTR_WAIT);
/* Get the handle to the canvas the user is currently working on. */
         HwndCanvas = PluginRuntime->HwndCanvas;
/* Call the ColorWorks function which displays the floated image in the preview
   window one row at a time.  The last parameter passed to this function is
   a pointer to a callback function which applies the effect to each row
   before it is placed in the preview window. */
         (*(PFN)PluginRuntime->CallbackFunctions.DisplayPreview)(HwndClient, HwndCanvas, ApplyMotionBlurToPreview);
/* Display the arrow pointer. */
         LoadSystemPointer(SPTR_ARROW);
         break;

      default: break;
   }
   return( WinDefWindowProc(HwndClient, message, param1, param2) );
}


/**************************************************************************
   This function applies the motion blur effect to the single row of float
   data passed in through "RowDataPointer".  This function is called by
   the ColorWorks callback function "DisplayPreview" just before it places a
   row of float data to the preview window, allowing the motion blur effect to
   be applied to the row.  When the preview windows dialog procedure recieves
   a WM_PAINT message it calles the ColorWorks function "DisplayPreview".
   The job of "DisplayPreview" is to display the floated image, one row at a
   time, in the preview window. For the preview to be of any value the effect
   must be applied to the data being displayed in the preview.  This function
   is called once for each row of data to be placed in the preview window,
   thus applying the effect to the preview.
**************************************************************************/
VOID ApplyMotionBlurToPreview(HWND HwndClient, UCHAR *RowDataPointer,
                              INT Y, INT PointsInDataPointerArray)
{
   struct _PluginRuntime *PluginRuntime;
   INT    FloatWidth, FloatHeight;
   HWND   HwndFrame, HwndDialog;
   float  ScaleX;
   float  ScaleY;
   SWP    swp;


/* Get the handle to the parent window of the preview window, which happens to
   be the frame window. */
   HwndFrame  = WinQueryWindow(HwndClient, QW_PARENT);
/* Get the handle to owner of the preview windows frame, put simply get the
   handle to the dialog window. */
   HwndDialog = WinQueryWindow(HwndFrame,  QW_OWNER);
/* Get the pointer to the PluginRuntime structure. */
   PluginRuntime = (struct _PluginRuntime *)WinQueryWindowPtr(HwndDialog, QWL_USER);
/* Query the dimensions of the preview window.*/
   WinQueryWindowPos(HwndClient, &swp);

/* Get the width and height of the floated image. */
   FloatWidth  = PluginRuntime->FloatData.Width;
   FloatHeight = PluginRuntime->FloatData.Height;

/* Compute the scale factors needed to display the float in the preview.  When
   a float is displayed in a preview window it will almost always need to be
   scaled up or down in order to fit into the window. */
   ScaleX = (float)FloatWidth  / (float)swp.cx;
   ScaleY = (float)FloatHeight / (float)swp.cy;

/* Apply the motion blur to the row of float data. */
   MotionBlurThePreview(HwndClient, RowDataPointer, Y, PointsInDataPointerArray,
                        ScaleX, ScaleY, PluginRuntime, HwndDialog);
}


VOID MotionBlurThePreview(HWND HwndClient, UCHAR *RowDataPointer, INT Y,
                          INT PointsInDataPointerArray, float ScaleX, float ScaleY,
                          struct _PluginRuntime *PluginRuntime, HWND HwndDialog)
{
   UCHAR *RowPointer, *PixelPointer;
   INT   FloatWidth, FloatHeight;
   INT   FloatRowsizeInBytes;
   INT   Red, Green, Blue, n;
   INT   YOffset, XOffset, jo;
   INT   x, y, i;
   UCHAR *FloatedImage;
   CHAR  BlurVelocity;
   INT   HorizontalDirection, VerticalDirection;

   FloatWidth          = PluginRuntime->FloatData.Width;
   FloatHeight         = PluginRuntime->FloatData.Height;
   FloatRowsizeInBytes = PluginRuntime->FloatData.RowsizeInBytes;
   FloatedImage        = PluginRuntime->FloatData.ImageDataPointer;
   YOffset             = (float)Y * ScaleY;

   BlurVelocity = (char)QueryNumericSpinButton(HwndDialog, VELOCITY_SPINNER);
   GetMotionBlurDirection(HwndDialog, &HorizontalDirection, &VerticalDirection);
   if(VerticalDirection*HorizontalDirection == MOVEMENT_IN_BOTH_DIRECTIONS)
      BlurVelocity = BlurVelocity * 10 / 14;

   for(i=0; i<PointsInDataPointerArray; i++)
   {
      XOffset = (float)i * ScaleX;
      Red = 0; Green = 0; Blue = 0; n = BlurVelocity;
      for(jo=0; jo<BlurVelocity; jo++)
      {
         x = XOffset + HorizontalDirection * jo;
         y = YOffset + VerticalDirection   * jo;
         if(y >= 0 && y < FloatHeight && x >= 0 && x < FloatWidth)
         {
            RowPointer = FloatedImage + (y * FloatRowsizeInBytes);
            PixelPointer = RowPointer + (x * BYTES_PER_PIXEL);
         }
         Blue  += n * *PixelPointer;
         Green += n * *(PixelPointer + 1);
         Red   += n * *(PixelPointer + 2);
         n--;
      }
      Blue  = Blue  / ((BlurVelocity+1) * BlurVelocity / 2);
      Green = Green / ((BlurVelocity+1) * BlurVelocity / 2);
      Red   = Red   / ((BlurVelocity+1) * BlurVelocity / 2);

      CHECKPIXELBOUNDS(Blue);
      CHECKPIXELBOUNDS(Green);
      CHECKPIXELBOUNDS(Red);

      if(IsPixelTransparant(&(RowDataPointer[i*BYTES_PER_PIXEL]), PluginRuntime) == NO)
      {
         ScrubColor(&Blue, &Green, &Red, PluginRuntime);
         RowDataPointer[i*BYTES_PER_PIXEL]     = Blue;
         RowDataPointer[(i*BYTES_PER_PIXEL)+1] = Green;
         RowDataPointer[(i*BYTES_PER_PIXEL)+2] = Red;
      }
   }
}
