#define INCL_WIN
#define INCL_ERRORS
#define INCL_BASE
#include <os2.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>


/****************************************************************************
   This header file contains all pragmas, function prototypes, defines, and
   structure definitions.
****************************************************************************/
#include "aquire.h"

/****************************************************************************
   This is the plugins initilization function.  It is called once, when
   ColorWorks is first started.  This function must appear in your plugin.
   The "EffectEntryPoint", "EffectName", and the "PluginCatagory" members of
   the "PluginInitilization" structure MUST! be filled in by this function.
   Allocating a static data area is optional.   A return of anything but
   "NO_ERROR" from this function indicates an error occured initilizing the
   plugin.  If an error is returned from this function, the plugin will not be
   made available to the user.  It is the responsibility of the plugin to
   inform the user that an error occured initilizing the plugin.
   Take special note of the second parameter being passed to the "WinLoadString"
   function.  This is the handle to the DLL being initilized.  When
   loading resources, such as a string, make sure you load the resource
   from the DLL using the DLL's handle. If you dont, and you use NULLHANDLE
   instead, the resource will attempt to be loaded from the .EXE instead of
   from your DLL. An ERROR for sure!  Notice this functions calling
   convention is "_System", this is because this function is called from
   ColorWorks by pointer refrence.
****************************************************************************/
INT _System InitilizePlugin(struct _PluginInitilization *PluginInitilization)
{
   CHAR   ErrorString[CCHMAXPATH];
   struct _AquireDialogSettings *AquireDialogSettings;

 /* This is an aquire plugin and belongs in the "Aquire" submenu off
    the "File" menu.  ColorWorks needs to be informed of the catagory of
    a particular plugin DLL in order to place it in the correct menu and
    properly process the plugin when selected by the user. See structure
    definition for additional details. */
   PluginInitilization->PluginCatagory = PLUGINAQUIRE;

 /* This is the name of the plugins entry point function, this is the function
    ColorWorks will call when the user selects this plugin from the menu. */
   strcpy(PluginInitilization->EffectEntryPoint, "PluginEntryPoint");

 /* This is the name of the plugin as it appears in the ColorWorks menu. */
   strcpy(PluginInitilization->EffectName, "Aquire Example...");

 /* Each time a user enters the aquire example dialog the settings of the dialogs
    controls should reflect the state of the controls the last time the user
    used the aquire example function.  This is simply good user interface design.
    To facilitate this the plugin interface allows the plugin designer to
    allocate an area of memory during initilization of the plugin.  This area
    of memory will be maintained by ColorWorks and will be made available to the
    plugin each time it is called.  The "StaticData" member of the
    "RuntimeStructure" structure is where the memory pointer will be placed for
    the plugins use. This memory can be used for any purpose the plugin designer
    sees fit. The next three lines of code allocate this static data area,
    loads its members with default values, and then sets the "StaticData"
    member of the "PluginInitilization" structure to point to the memory. */
   AquireDialogSettings = (struct _AquireDialogSettings *)malloc(sizeof(struct _AquireDialogSettings));
   if(AquireDialogSettings == (struct _AquireDialogSettings *)NULL)
   {
      WinLoadString(PluginInitilization->HwndAB, PluginInitilization->ModuleHandle, STRING1, sizeof(ErrorString), ErrorString);
      WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PSZ)ErrorString, (PSZ)NULL,
                    (USHORT)NULL, MB_ERROR | MB_OK | MB_MOVEABLE | MB_APPLMODAL);
      return(ERROR);
   }
   AquireDialogSettings->Resolution  = RES_200;
   PluginInitilization->StaticData = (void *)AquireDialogSettings;

   return(NO_ERROR);
}

/****************************************************************************
   This is the function ColorWorks calls when the user selects the plugin
   aquire "Aquire Example..." from the ColorWorks menu.  The one parameter is a
   pointer to the "_ImpExpPluginRuntime" structure.  This structure contains the
   information necessary to perform the effect.  Notice this functions calling
   convention is "_System", this is because this function is called from
   ColorWorks by pointer refrence.
****************************************************************************/
VOID _System PluginEntryPoint(struct _ImpExpPluginRuntime *ImpExpPluginRuntime)
{
   LoadAquireDialog(ImpExpPluginRuntime);
}

/**************************************************************************
   This function loads the aquire example dialog.  Take special note of the
   fourth parameter being passed to the "WinDlgBox" function.  This is
   the handle to the DLL being used to execute the effect.  When loading
   resources, such as a dialog, make sure you load the resource from the DLL
   using the DLL's handle. If you dont, and you use NULLHANDLE instead, the
   resource will attempt to be loaded from the .EXE instead of from your DLL.
   An ERROR for sure!
**************************************************************************/
VOID LoadAquireDialog(struct _ImpExpPluginRuntime *ImpExpPluginRuntime)
{
   ULONG   rc;
   HMODULE DLLModuleHandle;
   HWND    HwndCWClientWindow;

   HwndCWClientWindow = ImpExpPluginRuntime->HwndCWClientWindow;
   DLLModuleHandle    = ImpExpPluginRuntime->ModuleHandle;
   rc = WinDlgBox(HWND_DESKTOP,    HwndCWClientWindow, (PFNWP)AquireDialogProcedure,
                  DLLModuleHandle, AQUIRE_DIALOG,  (PVOID)ImpExpPluginRuntime);
}

/**************************************************************************
   This is the dialog procedure for the aquire example dialog.  This
   function is mostly a dispatch routine.  It does perform a small amount
   of processing for some messages, but nothing big.
**************************************************************************/
MRESULT EXPENTRY AquireDialogProcedure(HWND   HwndDialog, ULONG  message,
                                       MPARAM param1,     MPARAM param2)
{
   struct _ImpExpPluginRuntime *ImpExpPluginRuntime;
   struct _AquireDialogSettings *AquireDialogSettings;

   ImpExpPluginRuntime = (struct _ImpExpPluginRuntime *)WinQueryWindowPtr(HwndDialog, QWL_USER);
   switch( message )
   {
      case WM_INITDLG :
         InitilizeAquireDialog(HwndDialog, message, param1, param2);
         return( (MRESULT)TRUE );

      case WM_CLOSE :
         message = WM_COMMAND;
         param1  = MPFROM2SHORT(DID_CANCEL, (SHORT2FROMMP(param1)));
         WinPostMsg(HwndDialog, message, param1, (MPARAM)NULL);
         return( (MRESULT)TRUE );

      case WM_COMMAND :
         switch(SHORT1FROMMP(param1))
         {
 /* The user clicked the "OK" pushbutton. */
            case DID_OK :
               WinDismissDlg(HwndDialog, (ULONG)NULL);
               LoadSystemPointer(SPTR_WAIT);
               AquireDialogSettings = (struct _AquireDialogSettings *)ImpExpPluginRuntime->StaticData;
               AquireDialogSettings->Resolution = QueryCanvasSizeRadioButtins(HwndDialog);
               CreateAquireCanvas(AquireDialogSettings, ImpExpPluginRuntime);
               LoadSystemPointer(SPTR_ARROW);
               return( (MRESULT)DID_OK );

 /* The user clicked the "Cancel" pushbutton. */
            case DID_CANCEL :
               WinDismissDlg(HwndDialog, (ULONG)NULL);
               return( (MRESULT)DID_CANCEL );
         }
         break;

      default: break;
   }
   return( WinDefDlgProc(HwndDialog, message, param1, param2));
}

/**************************************************************************
   This function initilizes the aquire example dialog, it is called when the
   aquire example dialog procedure recieves the "WM_INITDLG" message.  This
   function sets the dialogs window pointer to point to the "PluginRuntime"
   structure, sets the dialogs controls to their default values, and positions
   and activates the dialog.
**************************************************************************/
VOID InitilizeAquireDialog(HWND   HwndDialog, ULONG  message,
                           MPARAM param1,     MPARAM param2)
{
   INT    i;
   HWND   Hwnd;
   struct _ImpExpPluginRuntime *ImpExpPluginRuntime;
   struct _AquireDialogSettings *AquireDialogSettings;

   ImpExpPluginRuntime = (struct _ImpExpPluginRuntime *)param2;
 /* Get the pointer to the "AquireDialogSettings" structure. */
   AquireDialogSettings = (struct _AquireDialogSettings *)ImpExpPluginRuntime->StaticData;
 /* Set the user window pointer of the dialog to point to the "ImpExpPluginRuntime" structure. */
   WinSetWindowPtr(HwndDialog, QWL_USER, (PVOID)ImpExpPluginRuntime);

 /* Activate the correct radiobutton. */
   if(AquireDialogSettings->Resolution == RES_200)
      WinSendDlgItemMsg(HwndDialog, RES_200, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(AquireDialogSettings->Resolution == RES_256)
      WinSendDlgItemMsg(HwndDialog, RES_256, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(AquireDialogSettings->Resolution == RES_512)
      WinSendDlgItemMsg(HwndDialog, RES_512, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);
   if(AquireDialogSettings->Resolution == RES_750)
      WinSendDlgItemMsg(HwndDialog, RES_750, BM_SETCHECK, MPFROMSHORT(TRUE), (MPARAM)NULL);

 /* Center the Aquire example dialog on screen. */
   (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.CenterDialogBox)(HwndDialog);
 /* Set the aquire example as the active window. */
   WinSetActiveWindow(HWND_DESKTOP, HwndDialog);
}

/****************************************************************************
   This function creates a new canvas of the selected width and height.
   The function which fills the new canvas with a gradient is called, and
   then the client area of the new canvases window is invalidated to
   force a repaint.
****************************************************************************/
VOID CreateAquireCanvas(struct _AquireDialogSettings *AquireDialogSettings,
                        struct _ImpExpPluginRuntime *ImpExpPluginRuntime)
{
   INT Width, Height;
   struct _ImageInfoStruct ImageInfoStruct;

 /* Determine the size canvas the user wants created. */
   if(AquireDialogSettings->Resolution == RES_200)
      Width = Height = 200;
   if(AquireDialogSettings->Resolution == RES_256)
      Width = Height = 256;
   if(AquireDialogSettings->Resolution == RES_512)
      Width = Height = 512;
   if(AquireDialogSettings->Resolution == RES_750)
      Width = Height = 750;

 /* Create the canvas.  when this function returns the canvas will have been
    created and placed inside of ColorWorks' client area. ColorWorks is a
    24 bit image editor, all canvases created in ColorWorks are 24 bits per pel. */
   (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.ImpExpCreateCanvas)(Width, Height, "Aquire Example");

 /* This function fills the members of the ImageInfoStruct with values pretaining
    to the current canvas.  The "current canvas" is the canvas at the top of the
    Z-Order, or in this case the canvas we just created.  Because ColorWorks can
    have many canvases open at the same time we determin the canvas the user is
    currently working on by evaluating all canvases Z-Order and making current
    the one at the top. Whenever a canvas is created through a call to
    "ImpExpCreateCanvas" the newly created canvas is automaticaly made the
    current canvas. */
   (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.ImpExpGetImageInfo)(&ImageInfoStruct);

 /* Call to function which will fill the new canvas with a vertical gradient. */
   FillCanvasWithGradient(&ImageInfoStruct, ImpExpPluginRuntime);

 /* Invalidate the canvas in order to force a repaint.  Without this call the
    gradient which was just drawn to the canvas would not not visible. */
   WinInvalidateRect(ImageInfoStruct.HwndCanvasClient, NULL, (BOOL)FALSE);
}

/****************************************************************************
   This function fills the newly created canvas with a vertical gradient
   from black to white.  Depending on the size of the canvas which was
   created this gradient may repeat itself a number of times.
****************************************************************************/
VOID FillCanvasWithGradient(struct _ImageInfoStruct *ImageInfoStruct,
                            struct _ImpExpPluginRuntime *ImpExpPluginRuntime)
{
   INT   i;
   CHAR  *RowData;
   INT   Completion;
   UCHAR GradientValue = 0;
   INT   RedGammaTable[256], GreenGammaTable[256], BlueGammaTable[256];

 /* Allocate a block of memory as large as one row of image data. */
   RowData = (CHAR *)malloc(ImageInfoStruct->RowSizeInBytes);
   if(RowData == (CHAR *)NULL)
      return;

 /* This callback builds the gamma correction tables.  The gamma values used
    are the ones set in ColorWorks. */
  (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.ImpExpBuildGammaTables)
               (ImageInfoStruct->RedGamma, ImageInfoStruct->GreenGamma, ImageInfoStruct->BlueGamma,
                RedGammaTable, GreenGammaTable, BlueGammaTable);

 /* Process the entire canvas one row at a time, from the bottom of the canvas
    to the top. */
   for(i=0; i<ImageInfoStruct->Height; i++)
   {
 /* Calculate the precentage of the effect which has completed. */
      Completion = ((float)i / (float)ImageInfoStruct->Height) * 100.0;

 /* Set the completion ribbon to reflect the precentage of completion. */
      (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.SetCompletionRibbon)(Completion, ImageInfoStruct->HwndCanvas);

 /* Set "RowData" to the gradient value. Canvases in ColorWorks are all
    24 bits per pixel.  The color component layout is blue, green, red, with
    8 bits per component. */
      memset(RowData, GradientValue, ImageInfoStruct->RowSizeInBytes);

 /* This is a call to the function which will apply the gamma correction to the
    row data. */
     ApplyGammaCorrectionToData(RowData, RedGammaTable, GreenGammaTable, BlueGammaTable, ImageInfoStruct);

 /* Copy "RowData" to the canvas.  This function will take the contents of
    "RowData" and copy it to the current canvas at row location "i". */
      (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.ImpExpPutRow)(i, RowData, ImpExpPluginRuntime);

 /* Increment the "GradientValue".  The next row of image data drawn to the
    canvas will be one shade lighter than the previous row. */
      ++GradientValue;
   }

 /* The filling of th canvas with the vertical gradient is complete, so we.
   reset the completion ribbon. */
   (*(PFN)ImpExpPluginRuntime->ImpExpCallbackFunctions.SetCompletionRibbon)(TASK_COMPLETED, ImageInfoStruct->HwndCanvas);

 /* Free the memory block which was allocated with malloc. */
   free(RowData);
}


/****************************************************************************
   This function applies a gamma correction to the data which is going to
   be placed in the canvas. Gamma correction is a technique used to compensate
   for a monitors non linear response to voltage inputs.  Users set the
   gamma correction values in the "Canvas Settings" notebook in ColorWorks.
   These values are made available to the plugin through the callback
   function "ImpExpGetImageInfo". After obtaining these values, the plugin can
   then use the callback function "ImpExpBuildGammaTables" to build three
   gamma corrction tables which can be used, as in this function, to gamma
   correct the data.  It is strongly reccomended that you apply this gamma
   correction to the data. ColorWorks assume the correction was applied, and
   will reverse apply the correction when saving or printing the canvas.
   In addition, writers of export plugins are instructed to assume the gamma
   correction was applied, and should also reverse the application.
   So unless you have a very good reason not to do so, apply the gamma
   correction.

   NOTE: Depending on the settings of the gamma correction values in ColorWorks
         "Canvas Creation Settings" notebook, the gradient which is drawn in
         the canvas by this example might not be a pure gray ramp from black
         to white.
****************************************************************************/
VOID ApplyGammaCorrectionToData(CHAR *RowData, INT *RedGammaTable,
                                INT *GreenGammaTable, INT *BlueGammaTable,
                                struct _ImageInfoStruct *ImageInfoStruct)
{
   INT i, PixelLocation;

 /* Process every pixel in the row. */
   for(i=0; i<ImageInfoStruct->Width; i++)
   {
 /* Compute the location in the "RowData" array of the pixel to be corrected.
    Pixels are 3 bytes each, 24 bits, therefore the start of each pixels data
    is the pixels column number multiplied by 3. */
      PixelLocation = i * 3;
 /* Apply the gamma correction using a simple table lookup.  Fast and easy. */
      RowData[PixelLocation]   = BlueGammaTable[RowData[PixelLocation]];
      RowData[PixelLocation+1] = GreenGammaTable[RowData[PixelLocation+1]];
      RowData[PixelLocation+2] = RedGammaTable[RowData[PixelLocation+2]];
   }
}

/**************************************************************************
   This function querys and returns the checkstate of a radiobutton.
**************************************************************************/
USHORT IsRadioButtonChecked(HWND HwndDialog, ULONG RadioButtonID)
{
   USHORT CheckState;

   CheckState = (USHORT)WinSendDlgItemMsg(HwndDialog, RadioButtonID, BM_QUERYCHECK,
                                          (MPARAM)NULL, (MPARAM)NULL);
   return(CheckState);
}

/****************************************************************************
   This function querey the aquire dialog radio buttons and returns
   the define for the radio button which is currently checked.
****************************************************************************/
INT QueryCanvasSizeRadioButtins(HWND HwndDialog)
{
   USHORT CheckState;

 /* Is the "200 * 200" radiobutton checked? */
   CheckState = IsRadioButtonChecked(HwndDialog, RES_200);
   if(CheckState == CHECKED)
      return(RES_200);

 /* Is the "256 * 256" radiobutton checked? */
   CheckState = IsRadioButtonChecked(HwndDialog, RES_256);
   if(CheckState == CHECKED)
      return(RES_256);

 /* Is the "512 * 512" radiobutton checked? */
   CheckState = IsRadioButtonChecked(HwndDialog, RES_512);
   if(CheckState == CHECKED)
      return(RES_512);

 /* Is the "750 * 750" radiobutton checked? */
   CheckState = IsRadioButtonChecked(HwndDialog, RES_750);
   if(CheckState == CHECKED)
      return(RES_750);

   return(RES_200);
}

/**************************************************************************
   This function loads and sets an OS/2 system pointer.
**************************************************************************/
VOID LoadSystemPointer(LONG PointerID)
{
   HPOINTER HSystemPointer;

   HSystemPointer = WinQuerySysPointer(HWND_DESKTOP, PointerID, FALSE);
   WinSetPointer(HWND_DESKTOP, HSystemPointer);
}
