/* Include all header files used by dialog definitions. */
#include "aquir.h"

/* General defines used by the aquire plugin example. */
#define TASK_COMPLETED                     -1
#define NO                              FALSE
#define YES                              TRUE
#define CHECKED                             1
#define ERROR                      NO_ERROR-1
#define PLUGINAQUIRE                        1

/* Stringtable definitions. */
#define STRING1  1
#define STRING2  2

/*****************************************************************************
   This structure is passed to the plugins initilization funtion, "InitilizePlugin()".
   The plugins initilization function is called once, when ColorWorks is first
   started.   This structure is filled in by the plugin with information
   needed by ColorWorks in order to execute the plugin.
******************************************************************************/
typedef struct _PluginInitilization
        {
/* Data allocated by the plugins initilization.  This data is available to the
   aquire plugin for the entire ColorWorks session. */
           VOID  *StaticData;
/* Reserved, dont change contents of reserved variables. */
           CHAR  Reserved1[256];
/* This is the name of the function ColorWorks should call when the user
   selects your aquire plugin from the ColorWorks menu. */
           CHAR  EffectEntryPoint[100];
/* This is the name of your effect.  This name will appear in the ColorWorks
   menu, allowing the user to choose your effect by this name */
           CHAR  EffectName[100];
/* Reserved, dont change contents of reserved variables. */
           CHAR  Reserved2[100];
/* This is the handle to ColorWorks' anchor block. */
           HAB   HwndAB;
/* This is the module handle to the aquire plugin DLL. */
           HMODULE ModuleHandle;
/* This is the catagory of the plugin.  ColorWorks supports several types of
   plugins, each catagorie of plugin performs a different main task, and is
   placed in a different menu in the ColorWorks menu stucture. */
           CHAR    PluginCatagory;
        };

/***************************************************************************
   This structure is passed to the ColorWorks callback function
   "ImpExpGetImageInfo".  This function fills the members of this structure
   with the appropriate values.
****************************************************************************/
typedef struct _ImageInfoStruct
        {
/* This is the width of the image in pixels. */
           INT  Width;
/* This is the height of the image in pixels. */
           INT  Height;
/* This is the the size of one row of image data in bytes. */
           INT  RowSizeInBytes;
/* This is the handle to the canvas, this variable is needed by several of the
   ColorWorks callback functions. */
           VOID *HwndCanvas;
/* This is the handle to the canvases client window. */
           HWND HwndCanvasClient;
/* This is the name of the image. */
           CHAR Name[CCHMAXPATH];
/* This is the canvases red gamma value, as set in the canvas creation settings notebook. */
           double RedGamma;
/* This is the canvases green gamma value, as set in the canvas creation settings notebook. */
           double GreenGamma;
/* This is the canvases blue gamma value, as set in the canvas creation settings notebook. */
           double BlueGamma;
        };

/*****************************************************************************
   This structure contains the pointers to ColorWorks callback functions.
   These functions perform services which your plugin might want or need to use.
******************************************************************************/
typedef struct _ImpExpCallbackFunctions
        {
/* This is a pointer to the Colorworks function which is used to create a
   new canvas. */
           VOID *ImpExpCreateCanvas;
/* This is a pointer to the ColorWorks function which gets a row of data from
   the image. */
           VOID *ImpExpGetRow;
/* This is a pointer to the ColorWorks function which places a row of data in
   the image. */
           VOID *ImpExpPutRow;
/* This is a pointer to the Colorworks function which is used to get information
   about the canvas. */
           VOID *ImpExpGetImageInfo;
/* This is a pointer to the ColorWorks function which is used to set the canvas
   windows completion ribbon. */
           VOID *SetCompletionRibbon;
/* This is a pointer to the ColorWorks function which may be used to center any
   dialog boxes your plugin may use. */
           VOID *CenterDialogBox;
/* This is a pointer to the ColorWorks function which builds a red, green, and blue
   gamma correction table. */
           VOID *ImpExpBuildGammaTables;
/* This is a pointer to the ColorWorks function which builds a reverse red, green, and blue
   gamma correction table. */
           VOID *ImpExpBuildReverseGammaTables;
/* The next ten variables are reserved, dont alter the contents of reserved variables. */
           VOID *Reserved1;
           VOID *Reserved2;
           VOID *Reserved3;
           VOID *Reserved4;
           VOID *Reserved5;
           VOID *Reserved6;
           VOID *Reserved7;
           VOID *Reserved8;
           VOID *Reserved9;
           VOID *Reserved10;
        };

/*****************************************************************************
   This structure is passed to your plugins entry point procedure, in this
   exmple that procedure is "PluginEntryPoint()".  This structure contains
   information needed by your plugin in order to perform its task.
******************************************************************************/
typedef struct _ImpExpPluginRuntime
        {
/* This structure contains the pointers to callback functions. These functions
   perform services your aquire plugin might want or need to use.*/
           struct  _ImpExpCallbackFunctions ImpExpCallbackFunctions;
/* This is the data area allocated by your aquire plugins initilization function. */
           VOID    *StaticData;
/* This the number of CPUs in the system and available for ColorWorks' use. */
           CHAR    CPUCount;
/* This is the handle to ColorWorks' client window. */
           HWND    HwndCWClientWindow;
/* This is the handle to ColorWorks' frame window. */
           HWND    HwndCWFrameWindow;
/* This is the module handle to the aquire plugin DLL. */
           HMODULE ModuleHandle;
/* This is the catagory id assigend to the aquire plugin DLL by ColorWorks.*/
           CHAR    PluginCatagory;
/* This is the handle to ColorWorks' anchor block. */
           HAB     HwndAB;
        };

/*****************************************************************************
   This structure contains the current settings of the Aquire Example dialog.
*****************************************************************************/
typedef struct _AquireDialogSettings
        {
           INT  Resolution;
        };


/* Function prototypes */
INT  _System InitilizePlugin(struct _PluginInitilization *PluginInitilization);
VOID _System PluginEntryPoint(struct _ImpExpPluginRuntime *ImpExpPluginRuntime);
VOID LoadAquireDialog(struct _ImpExpPluginRuntime *ImpExpPluginRuntime);
MRESULT EXPENTRY AquireDialogProcedure(HWND   HwndDialog, ULONG  message,
                                       MPARAM param1,     MPARAM param2);
VOID InitilizeAquireDialog(HWND   HwndDialog, ULONG  message,
                           MPARAM param1,     MPARAM param2);
VOID LoadSystemPointer(LONG PointerID);
USHORT IsRadioButtonChecked(HWND HwndDialog, ULONG RadioButtonID);
INT QueryCanvasSizeRadioButtins(HWND HwndDialog);
VOID CreateAquireCanvas(struct _AquireDialogSettings *AquireDialogSettings,
                        struct _ImpExpPluginRuntime *ImpExpPluginRuntime);
VOID FillCanvasWithGradient(struct _ImageInfoStruct *ImageInfoStruct,
                            struct _ImpExpPluginRuntime *ImpExpPluginRuntime);
VOID ApplyGammaCorrectionToData(CHAR *RowData, INT *RedGammaTable,
                                INT *GreenGammaTable, INT *BlueGammaTable,
                                struct _ImageInfoStruct *ImageInfoStruct);
